/* source: https://jrl.ninja/etc/1/ */
body {
    font-family: Arial, Helvetica, sans-serif;
    background-color: #fffaf7;
    line-height: 1.3;
}
main {
    max-width: 38rem;
    padding: 1.5rem;
    margin: auto;
}
header {
    margin-bottom: 1.5rem;
}
h1 {
    margin-bottom: .5rem;
}
time {
    color: #888;
}
hr {
    border: none;
    height: 2px;
    background-color: #ddd;
    margin: 2rem auto;
}
footer {
    color: #888;
    margin-top: 2rem;
    text-align: center;
}
a {
    color: #ff3c3c;
    text-decoration: none;
    outline: 0;
}
a:hover {
    text-decoration: underline;
}
::selection {
    background-color: #fff888;
}

.inline {
    display: inline-block;
    margin: 2rem auto;
}

iframe {
    border: None;
}
